.page-wrapper{
    display: flex;
    flex-wrap: wrap;
    flex-direction: row;
    padding: 0;
}

.content-wrapper{
    display: flex;
    width: 100%;
    gap: 1rem;
}

.main-content{
    width: 100%;
    display: flex;
    flex-direction: column;
    gap: 1rem;
}


.variables-header{
    width: 100%;
    display:flex;
    flex-wrap: wrap;
    justify-content: space-between;
    align-items: center;
}

.ssb-title.variables-title{
    margin-top: 1rem;
    width: 100%;
}

.ssb-input{
    margin-bottom: 1rem;
}
.accordion-wrapper{
    display: flex;
    flex-direction: column;
    gap: 1rem;
}

.ssb-accordion.variable-accordion{
    border: 2px solid #62919A;
    padding: 0.5rem;
}

.ssb-accordion.variable-accordion > .accordion-body {
    padding: 1rem;
}

.variables-input-group{
    display: flex;
    flex-wrap: wrap;
    justify-content: flex-start;
    align-items: flex-start;
    gap: 1rem;
    padding-top: 1rem;
}

/*.input-section{
    margin: 1rem;
}*/

.alert-section{
    display: flex;
    flex-wrap: wrap;
    justify-content: space-around;
    gap: 1rem;
    padding-top: 1rem;
}

.ssb-title.input-section-title{
    border-bottom: 2px solid #62919A;
    margin-bottom: 1rem;

}

.ssb-input.variable-input{
    max-width: 500px;
    width: 100%;
}

.form-check.ssb-checkbox{
    align-self: center;
}

/*Target child selector (label) of variable-dropdown*/
.ssb-dropdown.variable-dropdown > .dropdown-label{
    margin-bottom: 0.6rem;
}
